#!/usr/bin/python3

import os
import logging
import configparser
import argparse
from argparse import RawDescriptionHelpFormatter
from bistu_wifi.bistu_wifi import logger
from bistu_wifi.bistu_wifi import login
from bistu_wifi.bistu_wifi import logout


if 'XDG_CONFIG_HOME' in os.environ:
    XDG_CONFIG_HOME = os.environ['XDG_CONFIG_HOME']
else:
    XDG_CONFIG_HOME = os.path.join(os.environ['HOME'], '.config')

DEFAULT_HOST = '10.1.206.13'
DEFAULT_CONFIG_LOCATION = os.path.join(XDG_CONFIG_HOME, 'bistu-wifi/auth.ini')


def main():

    parser = argparse.ArgumentParser(formatter_class=RawDescriptionHelpFormatter)

    parser.description = 'Login or logout bistu wifi, by sending a post request to authorization host'

    parser.epilog = 'The format of config file is like below\n\n' \
                    '    [DEFAULT]\n' \
                    '    username = 202000000\n' \
                    '    password = Bistu000000\n' \
                    '    host = 10.1.206.13\n'

    parser.add_argument('-u', '--username', nargs='?',
                        type=str, help='username you use to login bistu wifi')
    parser.add_argument('-p', '--password', nargs='?',
                        type=str, help='password you use to login bistu wifi')
    parser.add_argument('--host', nargs='?', default=None,
                        type=str, help='host to send login/logout request to, default use 10.1.206.13')
    parser.add_argument('-o', '--logout', action='store_true',
                        help='logout bistu wifi. try to login if not privided')
    parser.add_argument('-c', '--config', nargs='?', default=DEFAULT_CONFIG_LOCATION,
                        type=str, help='load info(username, password, host) from config file. '
                        f'Will try {DEFAULT_CONFIG_LOCATION} if not provided.')
    parser.add_argument('--notime', action='store_true',
                        help='don\'t output time field in log')

    args = parser.parse_args()

    log_level = logging.DEBUG if 'BISTU_WIFI_DEBUG' in os.environ else logging.INFO
    log_fmt = '%(levelname)s : %(funcName)s : %(message)s'
    if not args.notime:
        log_fmt = '%(asctime)s : ' + log_fmt

    logging.basicConfig(level=log_level, format=log_fmt)

    username = None
    password = None
    host = DEFAULT_HOST
    is_logout = False

    if args.config is not None:
        logger.info('loading info from config file %s ...', args.config)

        config = configparser.ConfigParser()
        config.read(args.config)

        d_section = config['DEFAULT']

        username = str(d_section['username']) if 'username' in d_section else username
        password = str(d_section['password']) if 'password' in d_section else password
        host     = str(d_section['host'])     if 'host'     in d_section else host

    username = args.username if args.username is not None else username
    password = args.password if args.password is not None else password
    host     = args.host     if args.host     is not None else host

    is_logout   = args.logout

    logger.info('parameter status: username = %s', username)
    logger.info('parameter status: password = %s', None if password is None else '*' * len(password))
    logger.info('parameter status: host     = %s', host)
    logger.info('parameter status: logout   = %s', is_logout)

    if is_logout:
        logout(host)
    else:
        if username is None or password is None:
            logger.error('cannot login wifi without username or password')
        else:
            login(username, password, host)


if __name__ == '__main__':
    main()
